"use strict";

exports.__esModule = true;
exports.PatternValidator = exports.MaxLengthValidator = exports.MinLengthValidator = exports.EmailValidator = exports.CheckboxRequiredValidator = exports.RequiredValidator = exports.MinValidator = exports.MaxValidator = void 0;

var _validators = require("../validators");

function toNumber(value) {
  return typeof value === "number" ? value : parseInt(value, 10);
}

class AbstractValidatorDirective {
  constructor() {
    this._validator = _validators.nullValidator;
    this._onChange = void 0;
    this.inputName = void 0;
  }

  handleChanges(changes) {
    if (this.inputName in changes) {
      const input = this.normalizeInput(changes[this.inputName].currentValue);
      this._validator = this.createValidator(input);

      if (this._onChange) {
        this._onChange();
      }
    }
  }

  validate(control) {
    return this._validator(control);
  }

  registerOnValidatorChange(fn) {
    this._onChange = fn;
  }

}

class MaxValidator extends AbstractValidatorDirective {
  constructor(...args) {
    super(...args);
    this.max = void 0;
    this.inputName = "max";

    this.normalizeInput = input => parseFloat(input);

    this.createValidator = max => (0, _validators.maxValidator)(max);
  }

  ngOnChanges(changes) {
    this.handleChanges(changes);
  }

}
/**
 * @description
 * Provider which adds `MinValidator` to the `NG_VALIDATORS` multi-provider list.
 */

/**
 * A directive which installs the {@link MinValidator} for any `formControlName`,
 * `formControl`, or control with `ngModel` that also has a `min` attribute.
 *
 * @see [Form Validation](guide/form-validation)
 *
 * @usageNotes
 *
 * ### Adding a min validator
 *
 * The following example shows how to add a min validator to an input attached to an
 * ngModel binding.
 *
 * ```html
 * <input type="number" ngModel min="4">
 * ```
 *
 * @ngModule ReactiveFormsModule
 * @ngModule FormsModule
 * @publicApi
 */


exports.MaxValidator = MaxValidator;

class MinValidator extends AbstractValidatorDirective {
  constructor(...args) {
    super(...args);
    this.min = void 0;
    this.inputName = "min";

    this.normalizeInput = input => parseFloat(input);

    this.createValidator = min => (0, _validators.minValidator)(min);
  }

  /**
   * Declare `ngOnChanges` lifecycle hook at the main directive level (vs keeping it in base class)
   * to avoid differences in handling inheritance of lifecycle hooks between Ivy and ViewEngine in
   * AOT mode. This could be refactored once ViewEngine is removed.
   * @nodoc
   */
  ngOnChanges(changes) {
    this.handleChanges(changes);
  }

}
/**
 * @description
 * An interface implemented by classes that perform asynchronous validation.
 *
 * @usageNotes
 *
 * ### Provide a custom async validator directive
 *
 * The following example implements the `AsyncValidator` interface to create an
 * async validator directive with a custom error key.
 *
 * ```typescript
 * import { of } from 'rxjs';
 *
 * @Directive({
 *   selector: '[customAsyncValidator]',
 *   providers: [{provide: NG_ASYNC_VALIDATORS, useExisting: CustomAsyncValidatorDirective, multi:
 * true}]
 * })
 * class CustomAsyncValidatorDirective implements AsyncValidator {
 *   validate(control: AbstractControl): Observable<ValidationErrors|null> {
 *     return of({'custom': true});
 *   }
 * }
 * ```
 *
 * @publicApi
 */


exports.MinValidator = MinValidator;

/**
 * @description
 * Provider which adds `RequiredValidator` to the `NG_VALIDATORS` multi-provider list.
 */

/**
 * @description
 * Provider which adds `CheckboxRequiredValidator` to the `NG_VALIDATORS` multi-provider list.
 */

/**
 * @description
 * A directive that adds the `required` validator to any controls marked with the
 * `required` attribute. The directive is provided with the `NG_VALIDATORS` multi-provider list.
 *
 * @see [Form Validation](guide/form-validation)
 *
 * @usageNotes
 *
 * ### Adding a required validator using template-driven forms
 *
 * ```
 * <input name="fullName" ngModel required>
 * ```
 *
 * @ngModule FormsModule
 * @ngModule ReactiveFormsModule
 * @publicApi
 */
class RequiredValidator {
  constructor() {
    this._required = false;
    this._onChange = void 0;
  }

  /**
   * @description
   * Tracks changes to the required attribute bound to this directive.
   */
  get required() {
    return this._required;
  }

  set required(value) {
    this._required = value != null && value !== false && `${value}` !== "false";
    if (this._onChange) this._onChange();
  }
  /**
   * Method that validates whether the control is empty.
   * Returns the validation result if enabled, otherwise null.
   * @nodoc
   */


  validate(control) {
    return this.required ? (0, _validators.requiredValidator)(control) : null;
  }
  /**
   * Registers a callback function to call when the validator inputs change.
   * @nodoc
   */


  registerOnValidatorChange(fn) {
    this._onChange = fn;
  }

}
/**
 * A Directive that adds the `required` validator to checkbox controls marked with the
 * `required` attribute. The directive is provided with the `NG_VALIDATORS` multi-provider list.
 *
 * @see [Form Validation](guide/form-validation)
 *
 * @usageNotes
 *
 * ### Adding a required checkbox validator using template-driven forms
 *
 * The following example shows how to add a checkbox required validator to an input attached to an
 * ngModel binding.
 *
 * ```
 * <input type="checkbox" name="active" ngModel required>
 * ```
 *
 * @publicApi
 * @ngModule FormsModule
 * @ngModule ReactiveFormsModule
 */


exports.RequiredValidator = RequiredValidator;

class CheckboxRequiredValidator extends RequiredValidator {
  /**
   * Method that validates whether or not the checkbox has been checked.
   * Returns the validation result if enabled, otherwise null.
   * @nodoc
   */
  //  override validate(control: AbstractControl): ValidationErrors|null {
  validate(control) {
    return this.required ? (0, _validators.requiredTrueValidator)(control) : null;
  }

}
/**
 * @description
 * Provider which adds `EmailValidator` to the `NG_VALIDATORS` multi-provider list.
 */

/**
 * A directive that adds the `email` validator to controls marked with the
 * `email` attribute. The directive is provided with the `NG_VALIDATORS` multi-provider list.
 *
 * @see [Form Validation](guide/form-validation)
 *
 * @usageNotes
 *
 * ### Adding an email validator
 *
 * The following example shows how to add an email validator to an input attached to an ngModel
 * binding.
 *
 * ```
 * <input type="email" name="email" ngModel email>
 * <input type="email" name="email" ngModel email="true">
 * <input type="email" name="email" ngModel [email]="true">
 * ```
 *
 * @publicApi
 * @ngModule FormsModule
 * @ngModule ReactiveFormsModule
 */


exports.CheckboxRequiredValidator = CheckboxRequiredValidator;

class EmailValidator {
  constructor() {
    this._enabled = false;
    this._onChange = void 0;
  }

  /**
   * @description
   * Tracks changes to the email attribute bound to this directive.
   */
  set email(value) {
    this._enabled = value === "" || value === true || value === "true";
    if (this._onChange) this._onChange();
  }
  /**
   * Method that validates whether an email address is valid.
   * Returns the validation result if enabled, otherwise null.
   * @nodoc
   */


  validate(control) {
    return this._enabled ? (0, _validators.emailValidator)(control) : null;
  }
  /**
   * Registers a callback function to call when the validator inputs change.
   * @nodoc
   */


  registerOnValidatorChange(fn) {
    this._onChange = fn;
  }

}
/**
 * @description
 * A function that receives a control and synchronously returns a map of
 * validation errors if present, otherwise null.
 *
 * @publicApi
 */


exports.EmailValidator = EmailValidator;

/**
 * @description
 * Provider which adds `MinLengthValidator` to the `NG_VALIDATORS` multi-provider list.
 */

/**
 * A directive that adds minimum length validation to controls marked with the
 * `minlength` attribute. The directive is provided with the `NG_VALIDATORS` multi-provider list.
 *
 * @see [Form Validation](guide/form-validation)
 *
 * @usageNotes
 *
 * ### Adding a minimum length validator
 *
 * The following example shows how to add a minimum length validator to an input attached to an
 * ngModel binding.
 *
 * ```html
 * <input name="firstName" ngModel minlength="4">
 * ```
 *
 * @ngModule ReactiveFormsModule
 * @ngModule FormsModule
 * @publicApi
 */
class MinLengthValidator {
  constructor() {
    this._validator = _validators.nullValidator;
    this._onChange = void 0;
    this.minlength = void 0;
  }

  // This input is always defined, since the name matches selector.

  /** @nodoc */
  ngOnChanges(changes) {
    if ("minlength" in changes) {
      this._createValidator();

      if (this._onChange) this._onChange();
    }
  }
  /**
   * Method that validates whether the value meets a minimum length requirement.
   * Returns the validation result if enabled, otherwise null.
   * @nodoc
   */


  validate(control) {
    return this.enabled() ? this._validator(control) : null;
  }
  /**
   * Registers a callback function to call when the validator inputs change.
   * @nodoc
   */


  registerOnValidatorChange(fn) {
    this._onChange = fn;
  }

  _createValidator() {
    this._validator = this.enabled() ? (0, _validators.minLengthValidator)(toNumber(this.minlength)) : _validators.nullValidator;
  }
  /** @nodoc */


  enabled() {
    return this.minlength != null
    /* both `null` and `undefined` */
    ;
  }

}
/**
 * @description
 * Provider which adds `MaxLengthValidator` to the `NG_VALIDATORS` multi-provider list.
 */

/**
 * A directive that adds max length validation to controls marked with the
 * `maxlength` attribute. The directive is provided with the `NG_VALIDATORS` multi-provider list.
 *
 * @see [Form Validation](guide/form-validation)
 *
 * @usageNotes
 *
 * ### Adding a maximum length validator
 *
 * The following example shows how to add a maximum length validator to an input attached to an
 * ngModel binding.
 *
 * ```html
 * <input name="firstName" ngModel maxlength="25">
 * ```
 *
 * @ngModule ReactiveFormsModule
 * @ngModule FormsModule
 * @publicApi
 */


exports.MinLengthValidator = MinLengthValidator;

class MaxLengthValidator {
  constructor() {
    this._validator = _validators.nullValidator;
    this._onChange = void 0;
    this.maxlength = void 0;
  }

  // This input is always defined, since the name matches selector.

  /** @nodoc */
  ngOnChanges(changes) {
    if ("maxlength" in changes) {
      this._createValidator();

      if (this._onChange) this._onChange();
    }
  }
  /**
   * Method that validates whether the value exceeds the maximum length requirement.
   * @nodoc
   */


  validate(control) {
    return this.enabled() ? this._validator(control) : null;
  }
  /**
   * Registers a callback function to call when the validator inputs change.
   * @nodoc
   */


  registerOnValidatorChange(fn) {
    this._onChange = fn;
  }

  _createValidator() {
    this._validator = this.enabled() ? (0, _validators.maxLengthValidator)(toNumber(this.maxlength)) : _validators.nullValidator;
  }
  /** @nodoc */


  enabled() {
    return this.maxlength != null
    /* both `null` and `undefined` */
    ;
  }

}
/**
 * @description
 * Provider which adds `PatternValidator` to the `NG_VALIDATORS` multi-provider list.
 */

/**
 * @description
 * A directive that adds regex pattern validation to controls marked with the
 * `pattern` attribute. The regex must match the entire control value.
 * The directive is provided with the `NG_VALIDATORS` multi-provider list.
 *
 * @see [Form Validation](guide/form-validation)
 *
 * @usageNotes
 *
 * ### Adding a pattern validator
 *
 * The following example shows how to add a pattern validator to an input attached to an
 * ngModel binding.
 *
 * ```html
 * <input name="firstName" ngModel pattern="[a-zA-Z ]*">
 * ```
 *
 * @ngModule ReactiveFormsModule
 * @ngModule FormsModule
 * @publicApi
 */


exports.MaxLengthValidator = MaxLengthValidator;

class PatternValidator {
  constructor() {
    this._validator = _validators.nullValidator;
    this._onChange = void 0;
    this.pattern = void 0;
  }

  // This input is always defined, since the name matches selector.

  /** @nodoc */
  ngOnChanges(changes) {
    if ("pattern" in changes) {
      this._createValidator();

      if (this._onChange) this._onChange();
    }
  }
  /**
   * Method that validates whether the value matches the pattern requirement.
   * @nodoc
   */


  validate(control) {
    return this._validator(control);
  }
  /**
   * Registers a callback function to call when the validator inputs change.
   * @nodoc
   */


  registerOnValidatorChange(fn) {
    this._onChange = fn;
  }

  _createValidator() {
    this._validator = (0, _validators.patternValidator)(this.pattern);
  }

}

exports.PatternValidator = PatternValidator;